/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is JIM.
 *
 * The Initial Developer of the Original Code is
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * services.highlevel.HLAccounts
 *
 *
 */
/**
 * HLAccounts
 *
 * Constructors:
 *   HLAccounts(jabberService)
 *
 * Method:
 *   getAccountInfo(accountName)
 *   setAccountInfo(accountName, username, password, resource, server, port)
 *   removeAccountInfo(accountName)
 *
 */

function HLAccounts(jabberService)
{
    this.jabberService = jabberService;

    this.jabberService.rdf.ns = JIM_NAMESPACE_URI+"rdf/accounts#";
    this.jabberService.rdf.getContainer("rdf:jim-accounts", "seq");
}

HLAccounts.prototype =
{
    getAccountInfo: function(accountName)
    {
        var account, ret, res;
        function vOn(v) { return v ? v.value : null };

        this.jabberService.rdf.ns = JIM_NAMESPACE_URI+"rdf/accounts#";

        res = this.jabberService.rdf.getLiteral(accountName);
        account = res.getSource("name");
        
        if (account)
            ret = {
                name:     accountName,
                username: vOn(account.getTarget("username")),
                password: vOn(account.getTarget("password")),
                resource: vOn(account.getTarget("resource")),
                server:   vOn(account.getTarget("server")),
                port:     vOn(account.getTarget("port")),
                useSSL:   vOn(account.getTarget("useSSL")) == "true"
            };
        return ret;
    },

    hasAccountInfo: function(accountName)
    {
        this.jabberService.rdf.ns = JIM_NAMESPACE_URI+"rdf/accounts#";
        return this.jabberService.rdf.getLiteral(accountName).
            getSource("name") != null;
    },

    setAccountInfo: function(accountName, username, password, resource, server, port, useSSL)
    {
        var account, container;

        this.jabberService.rdf.ns = JIM_NAMESPACE_URI+"rdf/accounts#";

        account = this.jabberService.rdf.getLiteral(accountName).
            getSource("name");

        if (!account)
            account = this.jabberService.rdf.getResource();

        account.setAssertion("name", accountName, true);
        account.setAssertion("resource", resource, true);
        account.setAssertion("username", username, true);

        if (password != null)
            account.setAssertion("password", password, true);
        else
            account.removeAssertion("password");

        account.setAssertion("server", server, true);
        account.setAssertion("port", port, true);
        if (useSSL)
            account.setAssertion("useSSL", "true", true);
        else
            account.removeAssertion("useSSL", null, false, true);

        container = this.jabberService.rdf.getContainer("rdf:jim-accounts");

        if (container.indexOf(account) == -1)
            container.appendElement(account);

    },

    removeAccountInfo: function(accountName)
    {
        var account;

        this.jabberService.rdf.ns = JIM_NAMESPACE_URI+"rdf/accounts#";

        account = this.jabberService.rdf.getLiteral(accountName).
            getSource("name");
        if (account) {
            this.jabberService.rdf.getContainer("rdf:jim-accounts").
                removeElement(account)
            account.remove();
        }
    }
}
