/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is JIM.
 *
 * The Initial Developer of the Original Code is
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * services.highlevel.HLAvatars
 *
 *
 */
ML.load("utils/observable.js");
ML.load("io/file.js");
ML.load("rdf/filerdf.js");
ML.load("utils/base64.js");
ML.load("utils/misc.js");

/**
 * HLAvatars
 *   Service which controls information about avatars
 *
 * Constructors:
 *   HLAvatars(jabberService) - constructor
 *
 * Events:
 *   onAvatarChanged(users, userid, resourceid)
 *   onAvatarThisChanged(thisUser)
 *
 */

function HLAvatars(jabberService)
{
    this.base = Observable;
    this.base();
    var nodes, node;
    var avatarsDir;
    var arr, i, rdf;

    avatarsDir = new File(jabberService.settingsDirectory, "avatars");
    if (!avatarsDir.exists)
        avatarsDir.createDirectory(0666);

    this.jabberService = jabberService;
    this.avatarsHash = { };
    this.avatarsDir = avatarsDir.path;

    nodes = jabberService.rdf.getContainer("rdf:jim-avatars", "seq").
        getElements();

    jabberService.rdf.ns = JIM_NAMESPACE_URI+"rdf/avatars#";

    while (nodes.hasNext()) {
        node = nodes.getNext();
        this.avatarsHash[node.getTarget("hash").value] =
            node.getTarget("url").value
    }

    rdf = new FileRDF("chrome://jim/content/data/avatars/avatars.rdf",
        JIM_NAMESPACE_URI+"rdf/avatars#");

    nodes = rdf.getContainer("rdf:jim-avatars").getElements();
    while (nodes.hasNext()) {
        node = nodes.getNext();
        this.avatarsHash[node.getTarget("hash").value] =
            node.getTarget("url").value
    }

    jabberService.core.iq.avatar.addObserver(this);
    jabberService.core.addObserver(this);
}

HLAvatars.prototype =
{
    __proto__: Observable.prototype,

    getAvatarByLongJID: function(longJID)
    {
        var resource = getResourceIDByLongJID(longJID);

        if (!resource)
            return null;

        resource = this.jabberService.localRDF.getResource(resource);
        resource = resorce.getTarget("avatar-uri", JIM_NAMESPACE_URI+"rdf/avatars#");

        return resource && resource.value;
    },

    getAvatarByResourceId: function(resource)
    {
        if (!resource)
            return null;

        resource = this.jabberService.localRDF.getResource(resource);
        resource = resource.getTarget("avatar-uri", JIM_NAMESPACE_URI+"rdf/avatars#");

        return resource && resource.value;
    },

    getThisAvatar: function()
    {
        return this.avatarUri;
    },

    setThisAvatar: function(uri)
    {
        var file, data, encodedData, hash;


        this.jabberService.localRDF.ns = JIM_NAMESPACE_URI+"rdf/avatars#";

        if (!url) {
            this.jabberService.core.highlevel.users.thisUser.
                setAssertion("avatar-uri", "", true);
            this.avatarUri = this.avatarData = this.avatarHash = null;
            return;
        }

        file = new File(url);

        if (!file.exists)
            throw "Plik nie istnieje";

        if (file.size > 8192)
            throw "Plik jest zbyt duy";

        file.open(file.MODE_RDONLY);
        
        data = file.read();
        hash = sha1(data);

        if (this.avatarUri && this.avatarsHash[hash] == this.avatarUri)
            return;

        this.avatarData = stringToBase64(data);
        this.avatarHash = hash;

        if (this.avatarsHash[hash]) {
            this.avatarUri = this.avatarsHash[hash];
            this.jabberService.core.highlevel.users.thisUser.
                setAssertion("avatar-uri", this.avatarUri, true);
        } else {
            this.avatarUri = this.addImageToCache(data, hash);
            this.jabberService.core.highlevel.users.thisUser.
                setAssertion("avatar-uri", this.avatarUri, true);
        }

        this.fireEvent("onAvatarThisChanged",
            this.jabberService.core.highlevel.users.thisUser);

        this.jabberService.core.highlevel.users.sendPresence();
    },

    setAvatarsHash: function(hash, file)
    {
        var res = this.jabberService.rdf.getResource();
        var uri = file.uri.spec;

        this.avatarsHash[hash] = uri;

        res.setAssertion("url", uri, true, JIM_NAMESPACE_URI+"rdf/avatars#");
        res.setAssertion("hash", hash, true, JIM_NAMESPACE_URI+"rdf/avatars#");
        this.jabberService.rdf.getContainer("rdf:jim-avatars").
            appendElement(res);

        return uri;
    },

    onIQAvatar: function(from, data)
    {
        var hash, url;
        var resource = this.jabberService.core.highlevel.users.
            getResourceIDByLongJID(from);

        if (!resource)
            return;

        setTimeout(this.__addImageToCache, 0, this, from, data,
            this.jabberService.localRDF.getResource(resource));
    },

    __addImageToCache: function(_this, from, data, resource)
    {
        data = base64ToString(data);
        hash = sha1(data);
        url = _this.avatarsHash[hash];

        if (!url)
            url = _this.addImageToCache(data, hash);

        resource.setAssertion("avatar-uri", url, true, JIM_NAMESPACE_URI+"rdf/avatars#");

        _this.fireEvent("onAvatarChanged",
            _this.jabberService.core.highlevel.users.users,
            resource.getTarget("resource-user", JIM_NAMESPACE_URI+"rdf/users#").value, resource.value);
    },

    addFileToCache: function(reader)
    {
        var data;

        reader.open();
        data = reader.read();
        reader.close();
        this.addImageToCache(data, sha1(data));
    },

    addImageToCache: function(data, hash)
    {
        var file;

        do {
            file = new File(this.avatarsDir, generateRandomName());
        } while (file.exists);

        file.open(file.MODE_WRONLY | file.MODE_CREATE);
        file.write(data);
        file.close();

        return this.setAvatarsHash(hash, file);
    },

    flushImageCache: function()
    {
        var container = jabberService.rdf.getContainer("rdf:jim-avatars");
        var nodes = container.getElements();

        while (nodes.hasNext()) {
            node = nodes.getNext();
            try {
                var f = new File(node.getTarget("url", JIM_NAMESPACE_URI+"rdf/avatars#").value);
                f.deleteFile();
                container.removeElement(node, null, true);
            } catch (ex) { }
        }
    },

    onPresence: function(tag)
    {
        var t = tag.getElementsByTagNameNS('jabber:x:avatar', 'x')[0];
        var hash, val;

        if (t && (t = t.getElementsByTagName("hash")[0])) {
            var from = tag.getAttribute('from');
            var resource = this.jabberService.core.highlevel.users.
                getResourceIDByLongJID(from);

            if (!resource)
                return;

            hash = domToText(t);

            resource = this.jabberService.localRDF.getResource(resource);
            this.jabberService.localRDF.ns = JIM_NAMESPACE_URI+"rdf/avatars#";

            val = resource.getTarget("avatar-uri");
            if (val && val.value == this.avatarsHash[hash])
                return;

            if (t = this.avatarsHash[hash]) {
                resource.setAssertion("avatar-uri", t, true);
            } else {
                resource.avatar = '';
                if (this.avatarsHash[hash] != '')
                    this.jabberService.core.iq.avatar.getAvatar(from);
                resource.setAssertion("avatar-uri", '', true);
            }
            this.fireEvent("onAvatarChanged",
                this.jabberService.core.highlevel.users.users,
                resource.getTarget("resource-user", JIM_NAMESPACE_URI+"rdf/users#").value,
                resource.value);
        }
    },

    onPresenceSend: function(tag)
    {
        if (this.avatarUri)
            tag.addElement("jabber:x:avatar", "x").
                addElement("jabber:x:avatar", "hash").
                    addText(this.avatarData);
    },
}
